#ifndef mozilla_dom_UnionConversions_h__
#define mozilla_dom_UnionConversions_h__

#include "XPCWrapper.h"
#include "mozilla/dom/UnionTypes.h"
#include "nsDOMQS.h"
#include "nsDebug.h"

namespace mozilla {
namespace dom {

class ArrayBufferOrLongArgument {
public:
  ArrayBufferOrLongArgument(const ArrayBufferOrLong& aUnion) : mUnion(const_cast<ArrayBufferOrLong&>(aUnion))
  {
  }

  bool TrySetToArrayBuffer(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    mArrayBufferHolder.construct(&value.toObject());
    if (!mArrayBufferHolder.ref().inited()) {
      tryNext = true;
      return true;
    }
    SetAsArrayBuffer() = mArrayBufferHolder.addr();
    return true;
  }

  bool TrySetToLong(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    if (!ValueToPrimitive<int32_t, eDefault>(cx, value, &SetAsLong())) {
      return false;
    }
    return true;
  }

private:
  NonNull<ArrayBuffer>& SetAsArrayBuffer()
  {
    mUnion.mType = mUnion.eArrayBuffer;
    return mUnion.mValue.mArrayBuffer.SetValue();
  }
  int32_t& SetAsLong()
  {
    mUnion.mType = mUnion.eLong;
    return mUnion.mValue.mLong.SetValue();
  }

  Maybe<ArrayBuffer> mArrayBufferHolder;

  ArrayBufferOrLong& mUnion;
};


class HTMLElementOrLongArgument {
public:
  HTMLElementOrLongArgument(const HTMLElementOrLong& aUnion) : mUnion(const_cast<HTMLElementOrLong&>(aUnion))
  {
  }

  bool TrySetToHTMLElement(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    jsval tmpVal = value;
    nsIDOMHTMLElement* tmp;
    if (NS_FAILED(xpc_qsUnwrapArg<nsIDOMHTMLElement>(cx, value, &tmp, static_cast<nsIDOMHTMLElement**>(getter_AddRefs(mHTMLElementHolder)), &tmpVal))) {
      if (mUnion.mType != mUnion.eUninitialized) {
        mUnion.DestroyHTMLElement();
      }tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    SetAsHTMLElement() = tmp;
    return true;
  }

  bool TrySetToLong(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    if (!ValueToPrimitive<int32_t, eDefault>(cx, value, &SetAsLong())) {
      return false;
    }
    return true;
  }

private:
  nsIDOMHTMLElement*& SetAsHTMLElement()
  {
    mUnion.mType = mUnion.eHTMLElement;
    return mUnion.mValue.mHTMLElement.SetValue();
  }
  int32_t& SetAsLong()
  {
    mUnion.mType = mUnion.eLong;
    return mUnion.mValue.mLong.SetValue();
  }

  nsRefPtr<nsIDOMHTMLElement> mHTMLElementHolder;

  HTMLElementOrLong& mUnion;
};


class HTMLImageElementOrHTMLCanvasElementOrHTMLVideoElementArgument {
public:
  HTMLImageElementOrHTMLCanvasElementOrHTMLVideoElementArgument(const HTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement& aUnion) : mUnion(const_cast<HTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement&>(aUnion))
  {
  }

  bool TrySetToHTMLImageElement(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    jsval tmpVal = value;
    nsHTMLImageElement* tmp;
    if (NS_FAILED(xpc_qsUnwrapArg<nsHTMLImageElement>(cx, value, &tmp, static_cast<nsHTMLImageElement**>(getter_AddRefs(mHTMLImageElementHolder)), &tmpVal))) {
      if (mUnion.mType != mUnion.eUninitialized) {
        mUnion.DestroyHTMLImageElement();
      }tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    SetAsHTMLImageElement() = tmp;
    return true;
  }

  bool TrySetToHTMLCanvasElement(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    jsval tmpVal = value;
    nsHTMLCanvasElement* tmp;
    if (NS_FAILED(xpc_qsUnwrapArg<nsHTMLCanvasElement>(cx, value, &tmp, static_cast<nsHTMLCanvasElement**>(getter_AddRefs(mHTMLCanvasElementHolder)), &tmpVal))) {
      if (mUnion.mType != mUnion.eUninitialized) {
        mUnion.DestroyHTMLCanvasElement();
      }tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    SetAsHTMLCanvasElement() = tmp;
    return true;
  }

  bool TrySetToHTMLVideoElement(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    jsval tmpVal = value;
    nsHTMLVideoElement* tmp;
    if (NS_FAILED(xpc_qsUnwrapArg<nsHTMLVideoElement>(cx, value, &tmp, static_cast<nsHTMLVideoElement**>(getter_AddRefs(mHTMLVideoElementHolder)), &tmpVal))) {
      if (mUnion.mType != mUnion.eUninitialized) {
        mUnion.DestroyHTMLVideoElement();
      }tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    SetAsHTMLVideoElement() = tmp;
    return true;
  }

private:
  nsHTMLImageElement*& SetAsHTMLImageElement()
  {
    mUnion.mType = mUnion.eHTMLImageElement;
    return mUnion.mValue.mHTMLImageElement.SetValue();
  }
  nsHTMLCanvasElement*& SetAsHTMLCanvasElement()
  {
    mUnion.mType = mUnion.eHTMLCanvasElement;
    return mUnion.mValue.mHTMLCanvasElement.SetValue();
  }
  nsHTMLVideoElement*& SetAsHTMLVideoElement()
  {
    mUnion.mType = mUnion.eHTMLVideoElement;
    return mUnion.mValue.mHTMLVideoElement.SetValue();
  }

  nsRefPtr<nsHTMLImageElement> mHTMLImageElementHolder;
  nsRefPtr<nsHTMLCanvasElement> mHTMLCanvasElementHolder;
  nsRefPtr<nsHTMLVideoElement> mHTMLVideoElementHolder;

  HTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement& mUnion;
};


class HTMLOptionElementOrHTMLOptGroupElementArgument {
public:
  HTMLOptionElementOrHTMLOptGroupElementArgument(const HTMLOptionElementOrHTMLOptGroupElement& aUnion) : mUnion(const_cast<HTMLOptionElementOrHTMLOptGroupElement&>(aUnion))
  {
  }

  bool TrySetToHTMLOptionElement(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    jsval tmpVal = value;
    nsHTMLOptionElement* tmp;
    if (NS_FAILED(xpc_qsUnwrapArg<nsHTMLOptionElement>(cx, value, &tmp, static_cast<nsHTMLOptionElement**>(getter_AddRefs(mHTMLOptionElementHolder)), &tmpVal))) {
      if (mUnion.mType != mUnion.eUninitialized) {
        mUnion.DestroyHTMLOptionElement();
      }tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    SetAsHTMLOptionElement() = tmp;
    return true;
  }

  bool TrySetToHTMLOptGroupElement(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    jsval tmpVal = value;
    nsHTMLOptGroupElement* tmp;
    if (NS_FAILED(xpc_qsUnwrapArg<nsHTMLOptGroupElement>(cx, value, &tmp, static_cast<nsHTMLOptGroupElement**>(getter_AddRefs(mHTMLOptGroupElementHolder)), &tmpVal))) {
      if (mUnion.mType != mUnion.eUninitialized) {
        mUnion.DestroyHTMLOptGroupElement();
      }tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    SetAsHTMLOptGroupElement() = tmp;
    return true;
  }

private:
  nsHTMLOptionElement*& SetAsHTMLOptionElement()
  {
    mUnion.mType = mUnion.eHTMLOptionElement;
    return mUnion.mValue.mHTMLOptionElement.SetValue();
  }
  nsHTMLOptGroupElement*& SetAsHTMLOptGroupElement()
  {
    mUnion.mType = mUnion.eHTMLOptGroupElement;
    return mUnion.mValue.mHTMLOptGroupElement.SetValue();
  }

  nsRefPtr<nsHTMLOptionElement> mHTMLOptionElementHolder;
  nsRefPtr<nsHTMLOptGroupElement> mHTMLOptGroupElementHolder;

  HTMLOptionElementOrHTMLOptGroupElement& mUnion;
};


class ObjectOrLongArgument {
public:
  ObjectOrLongArgument(const ObjectOrLong& aUnion) : mUnion(const_cast<ObjectOrLong&>(aUnion))
  {
  }

  void SetToObject(JSObject* obj)
  {
    mUnion.mValue.mObject.SetValue() = obj;
    mUnion.mType = mUnion.eObject;
  }

  bool TrySetToLong(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    if (!ValueToPrimitive<int32_t, eDefault>(cx, value, &SetAsLong())) {
      return false;
    }
    return true;
  }

private:
  NonNull<JSObject>& SetAsObject()
  {
    mUnion.mType = mUnion.eObject;
    return mUnion.mValue.mObject.SetValue();
  }
  int32_t& SetAsLong()
  {
    mUnion.mType = mUnion.eLong;
    return mUnion.mValue.mLong.SetValue();
  }

  ObjectOrLong& mUnion;
};


class ObjectOrNullOrLongArgument {
public:
  ObjectOrNullOrLongArgument(const ObjectOrNullOrLong& aUnion) : mUnion(const_cast<ObjectOrNullOrLong&>(aUnion))
  {
  }

  bool SetNull()
  {
    mUnion.mType = mUnion.eNull;
    return true;
  }

  void SetToObject(JSObject* obj)
  {
    mUnion.mValue.mObject.SetValue() = obj;
    mUnion.mType = mUnion.eObject;
  }

  bool TrySetToLong(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    if (!ValueToPrimitive<int32_t, eDefault>(cx, value, &SetAsLong())) {
      return false;
    }
    return true;
  }

private:
  NonNull<JSObject>& SetAsObject()
  {
    mUnion.mType = mUnion.eObject;
    return mUnion.mValue.mObject.SetValue();
  }
  int32_t& SetAsLong()
  {
    mUnion.mType = mUnion.eLong;
    return mUnion.mValue.mLong.SetValue();
  }

  ObjectOrNullOrLong& mUnion;
};


class StringOrObjectArgument {
public:
  StringOrObjectArgument(const StringOrObject& aUnion) : mUnion(const_cast<StringOrObject&>(aUnion))
  {
  }

  bool TrySetToString(JSContext* cx, JSObject* scopeObj, const JS::Value& value, JS::Value* pvalue, bool& tryNext)
  {
    tryNext = false;
    if (!ConvertJSValueToString(cx, value, pvalue, eStringify, eStringify, mStringHolder)) {
      return false;
    }
    const_cast<NonNull<nsAString>&>(SetAsString()) = &mStringHolder;
    return true;
  }

  void SetToObject(JSObject* obj)
  {
    mUnion.mValue.mObject.SetValue() = obj;
    mUnion.mType = mUnion.eObject;
  }

private:
  NonNull<nsAString>& SetAsString()
  {
    mUnion.mType = mUnion.eString;
    return mUnion.mValue.mString.SetValue();
  }
  NonNull<JSObject>& SetAsObject()
  {
    mUnion.mType = mUnion.eObject;
    return mUnion.mValue.mObject.SetValue();
  }

  FakeDependentString mStringHolder;

  StringOrObject& mUnion;
};


} // namespace dom
} // namespace mozilla


#endif // mozilla_dom_UnionConversions_h__
